﻿// Adapted from https://gist.github.com/yorb/5445360
function get_vector_masks()
{
	app.bringToFront();
	
	// Constants
	var PIXEL_RATIO = app.activeDocument.resolution / 72; // Standardize pixels and points
	
	// Some helpers
	function cTID(s) { return charIDToTypeID(s); };
	function sTID(s) { return stringIDToTypeID(s); };
	
	// Work in pixels
	var OrigRulerUnits = app.preferences.rulerUnits;
	var OrigTypeUnits = app.preferences.typeUnits;
	app.preferences.rulerUnits = Units.PIXELS;
	app.preferences.typeUnits = TypeUnits.PIXELS;
 
	// Obtain the action manager indices of all the selected layers
	var SelIndices = GetSelectedLayersIdx();

	var collection = [];
	if (SelIndices.length == 1)
	{
		// Only a single layer is selected
		var mask = VectorMaskForActiveLayer();
		if (mask != undefined)
		{
			collection.push( {layerId: SelIndices[0], layerMask: mask} );
		}
	}
	else
	{
		// More than one layer is selected
		for (var i = 0; i < SelIndices.length; ++i)
		{
			if (MakeActiveByIndex(SelIndices[i], false) != -1)
			{
				var mask = VectorMaskForActiveLayer();
				if (mask != undefined)
				{
					collection.push( {layerId: SelIndices[i], layerMask: mask} );
				}
			}
		}
		
		// Restore original selection
		// SelectByIndices(SelIndices);
	}
	
	// Restore original ruler units
	app.preferences.rulerUnits = OrigRulerUnits;
	app.preferences.typeUnits = OrigTypeUnits;
 
	return collection;

	
	function VectorMaskForActiveLayer()
	{
		var doc = app.activeDocument;
		var PathItems = doc.pathItems;
	 
		// Nothing to do if the active layer has no path
		if (PathItems.length == 0)
			return;
			
		var dPathInfo = null;
		
		for (var i = 0; i < PathItems.length; ++i)
		{
			var Path = PathItems[i];
			
			// It would appear that the path for the selected layer is of kind VECTORMASK.  If so, when does WORKPATH come into play?
			if ((Path.kind == PathKind.VECTORMASK) || (Path.name == "Cropping Path"))
			{
				// Build a path info structure by copying the properties of this path, quantizing all anchor and left/right points
				var dPathInfo = BuildPathInfoFromPath(Path);
				if (dPathInfo.length > 0)
				{
					// We've now finished with the original path so it can go
					Path.remove();
					return dPathInfo;
				}
			}
		}
	}
	 
	 
	// Copies the specified path to a new path info structure, which is then returned.
	function BuildPathInfoFromPath(zInputPath)
	{
		// The output path will be an array of SubPathInfo objects
		var OutputPathInfo = [];
		
		// For each subpath in the input path
		for (var SubPathIndex = 0; SubPathIndex < zInputPath.subPathItems.length; ++SubPathIndex)
		{
			var InputSubPath = zInputPath.subPathItems[SubPathIndex];
	 
			var OutputPointInfo = [];    
			
			// For each point in the input subpath
			var NumPoints = InputSubPath.pathPoints.length;
			for (var PointIndex = 0; PointIndex < NumPoints; ++PointIndex)
			{
				var InputPoint = InputSubPath.pathPoints[PointIndex];
				
				var InputAnchor = InputPoint.anchor;
				var InputAnchorQ = InputAnchor;
				
				// Copy all the input point's properties to the output point info
				OutputPointInfo[PointIndex] = new PathPointInfo();
				OutputPointInfo[PointIndex].kind = InputPoint.kind;
				OutputPointInfo[PointIndex].anchor = InputPoint.anchor;
				OutputPointInfo[PointIndex].leftDirection = InputPoint.leftDirection;
				OutputPointInfo[PointIndex].rightDirection = InputPoint.rightDirection;
			}
	 
			// Create the SubPathInfo for our output path, and copy properties from the input sub path
			OutputPathInfo[SubPathIndex] = new SubPathInfo();
			OutputPathInfo[SubPathIndex].closed = InputSubPath.closed;
			OutputPathInfo[SubPathIndex].operation = InputSubPath.operation;
			OutputPathInfo[SubPathIndex].entireSubPath = OutputPointInfo;    
		}
	 
		return OutputPathInfo;
	}
	 
	 
	// Make a layer active by its action manager index
	function MakeActiveByIndex(zIndex, zVisible)
	{
		var desc = new ActionDescriptor();
		var ref = new ActionReference();
		ref.putIndex(cTID("Lyr "), zIndex)
		desc.putReference(cTID( "null"), ref );
		desc.putBoolean(cTID( "MkVs"), zVisible );
		executeAction(cTID( "slct"), desc, DialogModes.NO );
	}

	// Returns the action manager index of the selected layers
	function GetSelectedLayersIdx()
	{
		var selectedLayers = new Array;
		var ref = new ActionReference();
		ref.putEnumerated(cTID('Dcmn'), cTID('Ordn'), cTID('Trgt') );
		var desc = executeActionGet(ref);
	 
		if (desc.hasKey(sTID('targetLayers')))
		{
			desc = desc.getList(sTID('targetLayers'));
			var c = desc.count;
			var selectedLayers = new Array();
			for(var i = 0; i < c; ++i)
			{
				selectedLayers.push(desc.getReference(i).getIndex());
			}
		}
		else
		{
			var ref = new ActionReference(); 
			ref.putProperty(cTID('Prpr'), cTID('ItmI')); 
			ref.putEnumerated(cTID('Lyr '), cTID('Ordn'), cTID('Trgt'));
			selectedLayers.push(executeActionGet(ref).getInteger(cTID('ItmI')));
		}
	 
		return selectedLayers;
	}
	
	// Select layers by their action manager indices
	// function SelectByIndices(indices)
	// {
		// alert(indices);
		// var ref = new ActionReference();
		// var c = indices.length;
		// for(var i = 0; i < c; i++)
		// {
			// ref.putIdentifier(cTID('Lyr '), indices[i]);
		// }
		// var desc = new ActionDescriptor();
		// desc.putReference(cTID('null'), ref);
		// desc.putBoolean(cTID('MkVs'), false);
		// executeAction(cTID('slct'), desc, DialogModes.NO);
	// }
}
